package com.syc.slm.slmbi.config;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.ServletUtils;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.exception.SysException;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.Map;
import java.util.function.Supplier;

/**
 * mybatisplus自定义填充公共字段 ,即没有传的字段自动填充
 * @author lin
 * @date 2021-04-01 10:52:44
 */
@Component
@Slf4j
public class SysMetaObjectHandler implements MetaObjectHandler {
    private static final String ID = "id";
    private static final String APP_ID = "appId";
    private static final String CREATOR = "creator";
    private static final String CREATOR_NAME = "creatorName";
    private static final String CREATE_TIME = "createTime";
    private static final String UPDATER = "updater";
    private static final String UPDATER_NAME = "updaterName";
    private static final String UPDATE_TIME = "updateTime";
    private static final String RECORD_STATUS = "recordStatus";

    private CurrentUser userInfo(){
        HttpServletRequest request = ServletUtils.getHttpServletRequest();
        Map<String, String> headerMap = ServletUtils.getHeaderMap(request);
        if(CollectionUtils.isEmpty(headerMap)){
            throw new SysException("头部不能为空");
        }
        String token =request.getHeader(SlmConstants.TOKEN_PREFIX);
        if(StringUtils.isBlank(token)){
            throw new SysException("token不能为空");
        }
        Object currentUser = request.getAttribute(token);
        CurrentUser user =null;
        if(ObjectUtils.isNotEmpty(currentUser)){
            user= SlmConstants.gson.fromJson(currentUser.toString(),CurrentUser.class);
        }
        if(ObjectUtils.isEmpty(user)){
            throw new SysException("获取不到当前用户");
        }
        return user;
    }
    /**
     * 插入使用
     */
    @Override
    public void insertFill(MetaObject metaObject) {
        CurrentUser user = userInfo();
        fillValue(metaObject,
                  ID,
                  () -> (metaObject.getValue(ID) == null ? IdWorker.get32UUID() : metaObject.getValue(ID)));
        fillValue(metaObject,
                  APP_ID,
                  () -> (metaObject.getValue(APP_ID) == null ? user.getAppId() : metaObject.getValue(APP_ID)));
        fillValue(metaObject,
                  CREATOR,
                  () -> (metaObject.getValue(CREATOR) == null ? user.getUid() : metaObject.getValue(CREATOR)));
        fillValue(metaObject,
                  CREATOR_NAME,
                  () -> (metaObject.getValue(CREATOR_NAME) == null ? user.getName() : metaObject.getValue(CREATOR_NAME)));
        fillValue(metaObject,
                  CREATE_TIME,
                  () -> (metaObject.getValue(CREATE_TIME) == null ? new Date() : metaObject.getValue(CREATE_TIME)));
        fillValue(metaObject,
                  RECORD_STATUS,
                  () -> (metaObject.getValue(RECORD_STATUS) == null ? 1 : metaObject.getValue(RECORD_STATUS)));
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        CurrentUser user = userInfo();
        setFieldValByName(UPDATER, user.getUid(), metaObject);
        setFieldValByName(UPDATER_NAME, user.getName(),metaObject);
        setFieldValByName(UPDATE_TIME, new Date(), metaObject);
    }

    private void fillValue(MetaObject metaObject, String fieldName, Supplier<?> valueSupplier) {
        if (!metaObject.hasGetter(fieldName)) {
            return;
        }
        Object sidObj = metaObject.getValue(fieldName);
        if (sidObj == null && metaObject.hasSetter(fieldName) && valueSupplier != null) {
            setFieldValByName(fieldName, valueSupplier.get(), metaObject);
        }
    }

}
