package com.syc.slm.slmbi.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.syc.slm.common.core.util.R;
import com.syc.slm.slmbi.dto.BusinessDTO;
import com.syc.slm.slmbi.dto.BusinessStatisticsDTO;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.service.BusinessConfigService;
import com.syc.slm.slmbi.vo.BusinessConfigVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * slm-bi
 * 系统业务配置
 * @author : lin
 * @date : 2021-04-20 09:23
 **/
@RequestMapping("/{appId}/business")
@RestController
@Api(value = "系统业务配置接口", tags = "系统业务配置接口")
public class BusinessConfigController extends BaseRestController{

    @Autowired
    private BusinessConfigService businessConfigService;

    @GetMapping
    @ApiOperation("数据表查询")
    @ApiImplicitParam(name = "appId", value = "租户id",dataTypeClass = String.class)
    public R<List<BusinessConfigVo>> selectConfigList(@PathVariable String appId) {
        return call(()->(businessConfigService.selectConfigList()));
    }


    @GetMapping("/{id}")
    @ApiOperation("根据id进行数据表查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "appId", value = "租户id",dataTypeClass = String.class),
        @ApiImplicitParam(name = "id", value = "主键id",dataTypeClass = String.class)
    })
    public R<BusinessConfigVo> selectOneConfig(@PathVariable String appId,@PathVariable String id) {
        return call(()->(businessConfigService.selectOneConfig(id)));
    }

    @PostMapping("/filed")
    @ApiOperation("根据配置字段获取业务数据")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<Map<String,List<Object>>> getBusinessData(HttpServletRequest request, @PathVariable String appId,@RequestBody BusinessDTO dto) {
        return  call(()->{
            CurrentUser currentUser = getCurrentUser(request);
            return businessConfigService.getBusinessData(dto,dcClientHost+businessFiledData,currentUser);
        });
    }

    @PostMapping("/data/field/statistics")
    @ApiOperation("根据配置字段获取字段的统计数据")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class)
    public R<Map<String,Object>> listBusinessFieldStatistics(HttpServletRequest request,
        @PathVariable String appId,
        @RequestBody BusinessStatisticsDTO dto) {
        return call(() -> {
            CurrentUser currentUser = getCurrentUser(request);
            return businessConfigService.listBusinessFieldStatistics(dcClientHost+businessDataFieldStatistics,dto, currentUser);
        });
    }

    @GetMapping("/filed-value/{tableName}/{filed}")
    @ApiOperation("根据字段以及表名查询对应的字段")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true, dataTypeClass = String.class)
    public R<IPage<String>> filedValue(HttpServletRequest request,
        @PathVariable String appId, @PathVariable String tableName, @PathVariable String filed, @RequestParam(value = "current", defaultValue = "1") Integer current, @RequestParam(value = "size",defaultValue = "100") Integer size, @RequestParam(value = "name", required = false) String name) {
        return call(() -> {
            CurrentUser currentUser = getCurrentUser(request);
            return businessConfigService.filedValue(currentUser,tableName, filed,current,size,name,dcClientHost+filedValue);
        });
    }
}
