package com.syc.slm.slmbi.controller;

import com.syc.slm.common.core.util.R;
import com.syc.slm.slmbi.dto.InstrumentPanelConfigDTO;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.service.InstrumentPanelConfigService;
import com.syc.slm.slmbi.vo.InstrumentPanelConfigVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * slm-bi
 * 仪表板样式以及数据配置
 *
 * @author : lin
 * @date : 2021-03-25 19:09
 **/

@RequestMapping("/{appId}/panel-configs")
@RestController
@Api(value = "仪表板样式以及数据配置", tags = "仪表板样式以及数据配置")
public class InstrumentPanelConfigController extends BaseRestController  {

    @Autowired
    private InstrumentPanelConfigService instrumentPanelConfigService;

    @PostMapping
    @ApiOperation("保存仪表板样式以及数据配置")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<String> saveInstrumentPanelConfig(@PathVariable String appId, @RequestBody InstrumentPanelConfigDTO config) {
        return call(()-> instrumentPanelConfigService.saveConfig(appId, config));
    }

    @PostMapping("/publish")
    @ApiOperation("保存发布仪表板样式以及数据配置")
    @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)
    public R<String> publish(@PathVariable String appId, @RequestBody InstrumentPanelConfigDTO config) {
        return call(()-> instrumentPanelConfigService.publish(appId,config));
    }

    @GetMapping("/{panelId}")
    @ApiOperation("根据仪表板id获取仪表板配置")
    @ApiImplicitParams({@ApiImplicitParam(name = "panelId", value = "仪表板id",dataTypeClass = String.class),
        @ApiImplicitParam(name = "appId", value = "租户id", required = true,dataTypeClass = String.class)})
    public R<InstrumentPanelConfigVo> getConfigByPanelId(@PathVariable String appId,
        @PathVariable("panelId") String panelId, HttpServletRequest request) {
        return call(()->{
            CurrentUser currentUser = getCurrentUser(request);
            InstrumentPanelConfigVo config = instrumentPanelConfigService.getConfigByPanelId(panelId);
            config.setUser(currentUser);
            return config;
        });
    }

}
