package com.syc.slm.slmbi.handler;

import com.syc.slm.common.core.util.R;
import com.syc.slm.slmbi.exception.SysException;
import lombok.extern.slf4j.Slf4j;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.validation.BindException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import javax.servlet.http.HttpServletRequest;

/**
 * slm-bi
 * 全局业务异常处理
 * @author : lin
 * @date : 2021-04-16 17:34
 **/
@RestControllerAdvice
@Slf4j
public class SysExceptionHandler {

    	/**
    	 * 业务异常
    	 *
    	 * @param e exception
    	 * @return exception
    	 */
    	@ExceptionHandler(SysException.class)
    	public R<Object> handleSysException(SysException e, HttpServletRequest request) {
    		log.info(e.getMsg());
    		return R.bad(e.getMsg(),e.getDebugMsg());
    	}

	/**
	 * mybaits抛出的异常，以及SysMetaObjectHandler自定义抛出的异常
	 * @param e
	 * @return
	 */
	@ExceptionHandler(MyBatisSystemException.class)
	public R<Object> handleMyBatisSystemException(MyBatisSystemException e) {
		log.info(e.getMessage());
		return R.bad(e.getCause().getCause().getMessage(),e.getMessage());
	}


	/**
	 * 时间格式抛出的异常
	 * @param e
	 * @return
	 */
	@ExceptionHandler(BindException.class)
	public R<Object> handleBindException(BindException e) {
		String message = e.getMessage();
		log.info(message);
		if (message.contains("DateTimeFormat")){
			return R.bad("时间格式错误",null);
		}else {
			return R.bad("请按照提示填写格式",null);
		}
	}



}
