package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.dao.BusinessConfigMapper;
import com.syc.slm.slmbi.dto.BusinessDTO;
import com.syc.slm.slmbi.dto.BusinessStatisticsDTO;
import com.syc.slm.slmbi.entity.BaseEntity;
import com.syc.slm.slmbi.entity.BusinessConfig;
import com.syc.slm.slmbi.entity.CurrentUser;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.BusinessConfigService;
import com.syc.slm.slmbi.vo.BusinessConfigVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * slm-bi
 * 系统业务表配置
 * @author : lin
 * @date : 2021-04-20 09:30
 **/
@Service
public class BusinessConfigServiceImpl extends ServiceImpl<BusinessConfigMapper, BusinessConfig> implements BusinessConfigService {
    @Override
    @Transactional(readOnly = true)
    public List<BusinessConfigVo> selectConfigList() {
        LambdaQueryWrapper<BusinessConfig> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        List<BusinessConfig> configs = baseMapper.selectList(where);
        if(CollectionUtils.isNotEmpty(configs)){
            configs.forEach(x->{
                try {
                    if(StringUtils.isNotBlank(x.getField())) {
                        x.setField(URLDecoder.decode(x.getField(), StandardCharsets.UTF_8.name()));
                    }
                }catch (Exception e){
                    e.printStackTrace();
                    throw new SysException("请联系管理员，数据转化出错了");
                }
            });
        }

        return CollectionUtils.isNotEmpty(configs)?SlmConstants.gson.fromJson(SlmConstants.gson.toJson(configs),new TypeToken<List<BusinessConfigVo>>(){}.getType()):
            Lists.newArrayList();
    }

    @Override
    @Transactional(readOnly = true)
    public BusinessConfigVo selectOneConfig(String id) {
        BusinessConfig co = baseMapper.selectById(id);
        if (org.apache.commons.lang3.ObjectUtils.isEmpty(co)){
            throw new SysException("业务数据表不存在");
        }
        try {
            if(StringUtils.isNotBlank(co.getField())) {
                co.setField(URLDecoder.decode(co.getField(), StandardCharsets.UTF_8.name()));
            }
        }catch (Exception e){
            e.printStackTrace();
            throw new SysException("请联系管理员，数据转化出错了");
        }
        return ObjectUtils.isNotEmpty(co)?SlmConstants.gson.fromJson(SlmConstants.gson.toJson(co), BusinessConfigVo.class):null;
    }

    @Override
    @Transactional(readOnly = true)
    public Map<String,List<Object>> getBusinessData(BusinessDTO dto,String uri, CurrentUser user) {
        if(StringUtils.isBlank(dto.getTable())){
            throw new SysException("请传入对应的表名");
        }

        if(CollectionUtils.isEmpty(dto.getStatisticsList())){
            throw new SysException("请传入字段统计参数");
        }else{
            if(dto.getStatisticsList().stream().anyMatch(x -> StringUtils.isBlank(x.getField()))){
                throw new SysException("y轴配置字段不能为空");
            }
            if(dto.getStatisticsList().stream().anyMatch(x -> StringUtils.isBlank(x.getStatisticsType()))){
                throw new SysException("统计类型不能为空");
            }
        }
        if(StringUtils.isBlank(dto.getX())){
            throw new SysException("x轴维度不能为空");
        }

        if(CollectionUtils.isEmpty(dto.getWhere())){
            throw new SysException("where 条件不能为空");
        }
        dto.getWhere().forEach(x->{
            if(CollectionUtils.isEmpty(x.keySet())){
                throw new SysException("条件 and 或者or 都必须说明");
            }
            x.forEach((k,v)->{
                if(v.stream().anyMatch(y -> StringUtils.isBlank(y.getKey()))
                           || v.stream().anyMatch(y -> ObjectUtils.isEmpty(y.getValue()))
                || v.stream().anyMatch(y -> StringUtils.isBlank(y.getQueryType()))) {
                    throw new SysException("where 条件不能key value queryType 都不能为空");
                }
            });
        });

        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId", user.getAppId());
        R<Map<String,List<Object>>> result = RestTemplateUtils.post(uri, null,dto, new TypeToken<R<Map<String,List<Object>>>>() {}.getType(), uriVariables);
        return result.detach();
    }

    @Override
    public Map<String,Object> listBusinessFieldStatistics(String uri, BusinessStatisticsDTO dto, CurrentUser user) {
        if(StringUtils.isBlank(dto.getTable())){
            throw new SysException("请传入对应的表名");
        }

        if(CollectionUtils.isEmpty(dto.getStatisticsList())){
            throw new SysException("请传入字段统计参数");
        }else{
            if(dto.getStatisticsList().stream().anyMatch(x -> StringUtils.isBlank(x.getField()))){
                throw new SysException("y轴配置字段不能为空");
            }
            if(dto.getStatisticsList().stream().anyMatch(x -> StringUtils.isBlank(x.getStatisticsType()))){
                throw new SysException("统计类型不能为空");
            }
        }
        if(CollectionUtils.isEmpty(dto.getWhere())){
            throw new SysException("where 条件不能为空");
        }
        dto.getWhere().forEach(x->{
            if(CollectionUtils.isEmpty(x.keySet())){
                throw new SysException("条件 and 或者or 都必须说明");
            }
            x.forEach((k,v)->{
                if(v.stream().anyMatch(y -> StringUtils.isBlank(y.getKey()))
                   || v.stream().anyMatch(y -> ObjectUtils.isEmpty(y.getValue()))
                   || v.stream().anyMatch(y -> StringUtils.isBlank(y.getQueryType()))) {
                    throw new SysException("where 条件不能key value queryType 都不能为空");
                }
            });
        });

        Map<String, String> uriVariables = Maps.newHashMap();
        uriVariables.put("appId", user.getAppId());
        R<Map<String,Object>> result = RestTemplateUtils.post(uri, null,dto, new TypeToken<R<Map<String,Object>>>() {}.getType(), uriVariables);
        return result.detach();
    }

    @Override
    @Transactional(readOnly = true)
    public IPage<String> filedValue(CurrentUser currentUser,String tableName, String filed, Integer current, Integer size,String name,String uri) {
        if(StringUtils.isBlank(tableName)){
            throw new SysException("表名必填");
        }
        if(StringUtils.isBlank(filed)){
            throw new SysException("对应检索字段必填");
        }
        Map<String, Object> uriVariables = Maps.newHashMap();
        uriVariables.put("appId", currentUser.getAppId());
        uriVariables.put("filed", filed);
        uriVariables.put("tableName", tableName);
        uriVariables.put("current", current);
        uriVariables.put("size", size);
        uriVariables.put("name", name);
        IPage<String> page = new Page<>(current,size);
        R<Map<String,Object>> result = RestTemplateUtils.get(uri, new TypeToken<R<Map<String,Object>>>() {}.getType(), uriVariables);
        Map<String, Object> resultMaps = result.detach();
        if(CollectionUtils.isNotEmpty(resultMaps)){
            page.setTotal(new BigDecimal(resultMaps.get("total")+"").longValue());
            List<String> records = SlmConstants.gson.fromJson(SlmConstants.gson.toJson(resultMaps.get("records")), new TypeToken<List<String>>() {}.getType());
            page.setRecords(records);
        }else{
            page.setTotal(0L);
        }
        return page;
    }
}
