package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.config.DataCenterConfig;
import com.syc.slm.slmbi.dao.InstrumentPanelGroupInitMapper;
import com.syc.slm.slmbi.dao.InstrumentPanelTreeInitMapper;
import com.syc.slm.slmbi.entity.InstrumentPanelGroupInit;
import com.syc.slm.slmbi.entity.InstrumentPanelTreeInit;
import com.syc.slm.slmbi.enums.NodeType;
import com.syc.slm.slmbi.service.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

/**
 * @author: guangyi-liang
 * @date: 2021/5/20
 */
@Service
public class CompanyServiceImpl implements CompanyService {

    @Autowired
    private DataCenterConfig centerConfig;

    @Autowired
    private InstrumentPanelGroupInitMapper groupInitMapper;

    @Autowired
    private InstrumentPanelTreeInitMapper treeInitMapper;

    @Override
    public Set<String> findAll(String appId) {
        return this.findAllAppId(appId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean initGroupAndTree(String appId) {
        String name = "全部分组";
        Set<String> stringSet = this.findAllAppId(appId);
        for (String companyAppId:stringSet){
            LambdaQueryWrapper<InstrumentPanelGroupInit> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(InstrumentPanelGroupInit::getAppId,companyAppId);
            Integer count = groupInitMapper.selectCount(wrapper);
            if (count <= 0){
                InstrumentPanelGroupInit groupInit = this.initGroup(companyAppId,name);
                groupInitMapper.insert(groupInit);
                InstrumentPanelTreeInit treeInit = this.initTree(groupInit);
                treeInitMapper.insert(treeInit);
            }
        }
        return true;
    }

    private Set<String> findAllAppId(String appId){
        Map<String,String> map = Maps.newHashMap();
        map.put("appId",appId);
        R<Set<String>> result = RestTemplateUtils.get(centerConfig.getHost()+centerConfig.getCompanyAppId(), new TypeToken<R<Set<String>>>() {}.getType(),map);
        return result.detach();
    }

    private InstrumentPanelGroupInit initGroup(String appId,String name){
        InstrumentPanelGroupInit groupInit = new InstrumentPanelGroupInit();
        groupInit.setId(this.getUuid());
        groupInit.setName(name);
        groupInit.setAppId(appId);
        groupInit.setRecordStatus(SlmConstants.DATA_VALID);
        groupInit.setCreateTime(new Date());
        groupInit.setCreator("系统默认");
        groupInit.setCreatorName("系统默认");
        return groupInit;
    }

    private InstrumentPanelTreeInit initTree(InstrumentPanelGroupInit group){
        InstrumentPanelTreeInit treeInit = new InstrumentPanelTreeInit();
        treeInit.setAppId(group.getAppId());
        treeInit.setCreateTime(group.getCreateTime());
        treeInit.setCreator(group.getCreator());
        treeInit.setCreatorName(group.getCreatorName());
        treeInit.setFullId(group.getId());
        treeInit.setFullName(group.getName());
        treeInit.setId(group.getId());
        treeInit.setName(group.getName());
        treeInit.setRecordStatus(SlmConstants.DATA_VALID);
        treeInit.setNodeType(NodeType.GROUP.value);
        return treeInit;
    }

    private String getUuid(){
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

}
